/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.control.notification;

/**
 * Notifier objects can be used to notify a {@link NotificationHandler NotificationHandler} that some event
 * has happened, e.g. a number of messages have passed a certain port.<br>
 * <br>
 * - must have an owner of type {@link ikr.simlib.control.notification.MultiNotificationHandler}<br>
 * - passes calls of its own method <code>notifyHandler()</code> on to the responsible notification handler<br>
 * 
 * <p>
 * See {@link NotificationHandler NotificationHandler} for more details.
 * </p>
 */
public class Notifier {

   protected MultiNotificationHandler owner;

   public Notifier() {
      owner = null;
   }

   public void notifyHandler() {
      if (owner == null) {
         throw new NotificationException("No notification handler set");
      }

      owner.notify(this);
   }

   public MultiNotificationHandler getOwner() {
      return owner;
   }

   public void setOwner(MultiNotificationHandler owner) {
      this.owner = owner;
   }

   public boolean isRegistered() {
      return (owner != null);
   }
}
