/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.distributions.discrete;

import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.parameters.reflection.ReflectionConstructable;
import ikr.simlib.random.RandomNumberGenerator;

/**
 * Zipf Distribution ("Discrete Pareto Distribution")
 * 
 * <pre>
 *  p_k=C*k&circ;(-q), k=1..N, q&gt;1
 *  C: normalization constant, C=1/sum(k&circ;(-q),k=1..N)
 * </pre>
 * 
 */
public class ZipfDistribution extends ZipfMandelbrotDistribution {

   private static final String Q_VALUE = "Q";
   private static final String N_VALUE = "N";

   /** as required by {@link ReflectionConstructable} */
   public static ZipfDistribution createInstance(SimNode ownNode, Parameters pars, RandomNumberGenerator rng) {
      final double q = pars.get(ownNode, Q_VALUE).asDouble();
      final int n = pars.get(ownNode, N_VALUE).asInteger();
      return new ZipfDistribution(q, n, rng);
   }

   public ZipfDistribution(double pQ, int pN, RandomNumberGenerator rng) {
      super(pQ, pN, 1.0, rng);
   }

   public ZipfDistribution(double pQ, int pN) {
      this(pQ, pN, getDefaultRNG());
   }

   @Override
   public int next() {
      return super.next() + 1;
   }
}
