/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.entities.demultiplexers;

import ikr.simlib.exceptions.SimulationException;
import ikr.simlib.messages.Message;
import ikr.simlib.model.SimNode;
import ikr.simlib.ports.output.OutputPort;

import java.util.ArrayList;
import java.util.List;

/**
 * For <code>StdDemultiplexer</code> the number of output ports is submitted to
 * the constructor.
 */
public abstract class StdDemultiplexer extends Demultiplexer {

   protected List<OutputPort> outputPorts;

   public StdDemultiplexer(int noOfPorts, SimNode ownNode) {
      super(ownNode);

      outputPorts = new ArrayList<OutputPort>(noOfPorts);

      for (int i = 0; i < noOfPorts; i++) {
         addPort(OutputPort.OUTPUT_PORT_NAME + " " + (i + 1));
      }
   }

   public StdDemultiplexer(SimNode ownNode) {
      this(0, ownNode);
   }

   @Override
   public OutputPort getOutput(int index) {
      return outputPorts.get(index);
   }

   public OutputPort addPort(String portName) {
      final OutputPort port = new OutputPort(this, portName) {
         @Override
         protected Message handleGetMessage() {
            return StdDemultiplexer.this.handleGetMessage(this);
         }

         @Override
         protected boolean handleIsMessageAvailable() {
            return StdDemultiplexer.this.handleIsMessageAvailable(this);
         }
      };
      outputPorts.add(port);
      return port;
   }

   /**
    * Uses "output <++N>" as port name
    */
   public OutputPort addPort() {
      String portName = OutputPort.OUTPUT_PORT_NAME + " " + (outputPorts.size() + 1);
      OutputPort port = addPort(portName);
      return port;
   }

   public void removePort(String portName) {
      for (OutputPort p : outputPorts) {
         if (p.getLocalName().equals(portName)) {
            outputPorts.remove(p);
            return;
         }
      }

      throw new SimulationException("Unknown port: " + portName);
   }
}
