/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.entities.phases;

import ikr.simlib.events.time.Duration;
import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.parameters.reflection.ReflectionConstructable;

/**
 * <code>DInfiniteServer</code> is a standard infinite server component with a
 * constant service time that can be specified in the constructor. It can be
 * used to model delays of all kinds.
 * 
 * @see InfiniteServer
 */
public class DInfiniteServer extends InfiniteServer {

   private static final String SERVICE_TIME = "ServiceTime";

   /** as required by {@link ReflectionConstructable} */
   public static DInfiniteServer createInstance(SimNode ownNode, Parameters pars) {
      final Duration serviceTime = pars.get(ownNode, SERVICE_TIME).asInstance(Duration.class, ownNode.createChildNode(SERVICE_TIME), pars);
      return new DInfiniteServer(serviceTime, ownNode);
   }

   protected Duration serviceTime;

   public DInfiniteServer(Duration d, SimNode ownNode) {
      super(ownNode);

      serviceTime = d;
   }

   public void setServiceTime(Duration serviceTime) {
      this.serviceTime = serviceTime;
   }

   @Override
   protected Duration getNextServiceTime() {
      return serviceTime;
   }

}
