/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.entities.phases;

import ikr.simlib.events.time.Duration;
import ikr.simlib.messages.Message;
import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.parameters.reflection.ReflectionConstructable;

/**
 * UnitPhase uses a constant distribution per unit service time is dependent on
 * the length)
 */
public class UnitPhase extends Phase {

   private static final String SERVICE_TIME_PER_UNIT = "ServiceTimePerUnit";

   /** as required by {@link ReflectionConstructable} */
   public static UnitPhase createInstance(SimNode ownNode, Parameters pars) {
      final Duration serviceTimePerUnit = pars.get(ownNode, SERVICE_TIME_PER_UNIT).asInstance(Duration.class, ownNode.createChildNode(SERVICE_TIME_PER_UNIT), pars);
      return new UnitPhase(serviceTimePerUnit, ownNode);
   }

   protected Duration serviceTimePerUnit;

   public UnitPhase(Duration serviceTimePerUnit, SimNode ownNode) {
      super(ownNode);

      this.serviceTimePerUnit = serviceTimePerUnit;
   }

   public void setServiceTimePerUnit(Duration serviceTimePerUnit) {
      this.serviceTimePerUnit = serviceTimePerUnit;
   }

   @Override
   protected Duration getMessageProcessingDuration(Message msg) {
      return serviceTimePerUnit.multiplyBy(msg.getLength());
   }
}
