/*
 * Copyright (C) 2007-2014 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

/**
 * <h1>Queues</h1> Queues that are based on the base class
 * {@link ikr.simlib.entities.queues.QueuingEntity} which is derived from
 * {@link ikr.simlib.entities.Entity} can store any kind of message. For
 * convenience reasons {@link ikr.simlib.entities.queues.QueuingEntity} has four
 * static methods to create commonly used queuing types, namely
 * bounded/unbounded FIFO (first in first out) and bounded/unbounded LIFO (last
 * in first out). These static methods return a QueuingEntity and directly set
 * the appropriate QueuingDiscipline. <br>
 * The lengths of a queue is measured in units (information units, e. g. bytes).
 * A unit corresponds to the length specified in the member <code>length</code>
 * of {@link ikr.simlib.messages.Message} and is also the basis to determine
 * whether a bounded queue is full. Nevertheless, the length of a queue in
 * number of packets is also remembered by <code>List<></code> in which the
 * messages are stored. The different kinds of lengths can be retrieved by
 * <code>queue.getQDisc().getCurrentNumberOfUnits()</code> and
 * <code>queue.getQDisc().getCurrentNumberOfMessages()</code>, respectively.<br>
 * <br>
 * <ul>
 * <li>The communication with the preceding or succeeding model component is
 * processed via an input port (class {@link ikr.simlib.ports.input.InputPort})
 * or via an output port (class {@link ikr.simlib.ports.output.OutputPort}).
 * <li>In {@link ikr.simlib.entities.queues.QueuingEntity} implementing a
 * {@link ikr.simlib.algorithms.queuingDisciplines.BoundedFIFOQDisc} messages
 * that arrive at the input port are buffered including their arrival time with
 * the method <code>put()</code> (if there is enough room in the queue).
 * <li>Simultaneously a message for the succeeding model component is offered.
 * <li>If there is not enough buffer space in the queue to store the currently
 * arriving message, an {@link ikr.simlib.entities.queues.QueuingEntity} will
 * drop the message, whereas when hosting a
 * {@link ikr.simlib.algorithms.queuingDisciplines.BoundedLIFOQDisc} as many
 * messages from the head of the queue are dropped as necessary to store the
 * arriving message. In case it is larger than the maximum allowed queue size,
 * it is dropped immediately without deleting any messages from the queue.
 * <li>Upon request by the succeeding model component the method
 * <code>get()</code> is used to retrieve the next message from the queue.
 * <li>Statistics are kept on
 * <ul>
 * <li>the waiting time in reference to waiting messages
 * <li>the waiting time in reference to all passing messages (transfer time)
 * <li>the queue length in number of units
 * <li>the queue length in number of messages
 * <li>the loss probability
 * </ul>
 * </ul>
 */
package ikr.simlib.entities.queues;