/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.messages;

import java.util.Deque;
import java.util.LinkedList;

/**
 * LabelMessage provides the user with one label that can be used for any
 * purposes
 */
public class LabelMessage extends Message {

   protected Deque<Integer> labelStack;

   public LabelMessage(long length, int label) {
      super(length);
      labelStack = new LinkedList<Integer>();
      push(label);
   }

   public boolean hasLabel() {
      return (!labelStack.isEmpty());
   }

   public int getNumberOfLabels() {
      return labelStack.size();
   }

   public void push(int label) {
      labelStack.addFirst(label);
   }

   public int current() {
      return labelStack.peekFirst();
   }

   public int pop() {
      return labelStack.pollFirst();
   }

   public int getLabel() {
      return current();
   }

   public void setLabel(int label) {
      pop();
      push(label);
   }

   @Override
   public String toString() {
      return super.toString() + " Label: " + getLabel();
   }
}
