/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.messages.factories;

import ikr.simlib.factories.Factory;
import ikr.simlib.messages.Message;
import ikr.simlib.model.SimNode;
import ikr.simlib.parameters.Parameters;
import ikr.simlib.parameters.reflection.ReflectionConstructable;
import ikr.simlib.parameters.reflection.ReflectionConstructable2;

public class MessageFactory implements Factory<Message>, ReflectionConstructable2<SimNode, Parameters> {

   private static final String LENGTH = "Length";
   
   /** as required by {@link ReflectionConstructable} */
   public static MessageFactory createInstance(SimNode ownNode, Parameters pars) {
      long length = Message.DEFAULT_MESSAGE_LENGTH;
      if (pars.hasParameter(ownNode, LENGTH))
         length = pars.get(ownNode, LENGTH).asLong();
      return new MessageFactory(length);
   }

   private final long length;

   public MessageFactory() {
      this.length = Message.DEFAULT_MESSAGE_LENGTH;
   }

   public MessageFactory(long length) {
      this.length = length;
   }

   @Override
   public Message create() {
      return new Message(length);
   }
}
