/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.messages.modifiers;

import ikr.simlib.messages.IPPacket;
import ikr.simlib.messages.Message;
import ikr.simlib.ports.Port;
import ikr.simlib.ports.PortMonitor;

/**
 * This IP packet marker can be attached to a port in order to change the type
 * field of an IP packet when transferred via <code>GetMessage</code>.
 * 
 */
public class IPPacketTypeMarker implements PortMonitor {

   protected int type;

   public IPPacketTypeMarker(int type) {
      this.type = type;
   }

   @Override
   public void monitorGetMessage(Message message, Port port) {
      IPPacket packet = (IPPacket) message;
      packet.setTypeId(type);
   }

   @Override
   public void monitorIsMessageAvailable(boolean messageAvailable, Port port) {
   }

   @Override
   public void monitorMessageIndication(Message message, Port port) {
   }

   public void setTypeId(int type) {
      this.type = type;
   }

   public int getTypeId() {
      return type;
   }
}
