/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.meters;

/**
 * <h1>Meters</h1> The base class for meters is the class <code>Meter</code>.
 * The classes derived from <code>Meter</code> either measure at one point or
 * between two points and thus are called
 * {@link ikr.simlib.meters.OnePointMeter} and
 * {@link ikr.simlib.meters.TwoPointMeter}, respectively. It is the meters
 * responsibility to derive a metric from each message or from the flow of
 * messages, and provide the values of this metric to a statistic.
 * <ul>
 * <li>The class {@link ikr.simlib.meters.OnePointMeter} provides the method
 * <code>attachInput()</code> which connects a
 * {@link ikr.simlib.ports.PortMonitor} to a port. Whenever a message travels
 * accross that port, the abstract method <code>evaluateMessage()</code> in the
 * meter is called.</li>
 * <li>The class {@link ikr.simlib.meters.TwoPointMeter} provides the methods
 * <code>attachFromPort()</code> and <code>attachToPort()</code> each of which
 * connects a {@link ikr.simlib.ports.PortMonitor} to the specified port.
 * Whenever a message travels accross one of those ports, the abstract method
 * <code>evaluateMessageFromPort()</code> or
 * <code>evaluateMessageToPort()</code>, respectively, in the meter is called.</li>
 * </ul>
 * 
 * @see ikr.simlib.meters.OnePointMeter
 * @see ikr.simlib.meters.TwoPointMeter
 */
public abstract class Meter {

}
