/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.meters.count;

import ikr.simlib.messages.Message;
import ikr.simlib.meters.OnePointMeter;
import ikr.simlib.ports.Port;
import ikr.simlib.statistics.UnweightedSampleCollector;

/**
 * An {@link MessageCountMeter} can be attached to a port to count messages. The
 * {@link #useMessage(Message)} method can be overridden to count only those
 * messages that satisfy certain criteria, e.g., based on message content, or
 * message type.
 */
public class MessageCountMeter extends OnePointMeter {

   /**
    * creates a new {@link MessageCountMeter} with a statistic that accepts no
    * parameters
    * 
    * @param statistic
    *           statistic
    * @return the new {@link MessageCountMeter}
    */
   public static MessageCountMeter createWithVoidStatistic(UnweightedSampleCollector<Void> statistic) {
      return new MessageCountMeter(statistic);
   }

   /** statistic to update */
   private final UnweightedSampleCollector<Void> statistic;

   protected MessageCountMeter(UnweightedSampleCollector<Void> statistic) {
      this.statistic = statistic;
   }

   /**
    * called when a message was seen
    * 
    * @param msg
    *           the message
    */
   protected void evaluateSample(Message msg) {
      statistic.update(null);
   }

   /**
    * filters messages that should be used for counting
    * 
    * @param msg
    *           the observed message
    */
   protected boolean useMessage(Message msg) {
      return true;
   }

   @Override
   protected void evaluateMessage(Message msg, Port port) {
      if (useMessage(msg)) {
         evaluateSample(msg);
      }
   }

}
