/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.parameters.keys;

public class Key extends KeyQueryBase {

   public Key(String key) {
      super(key);
   }

   /**
    * all components must match
    */
   public boolean matches(Query query) {
      if (query.parts.length != parts.length) {
         // length doesn't match
         return false;

      } else {
         // check each part
         for (int i = 0; i < parts.length; i++) {
            if (!regexMatches(query.parts[i], parts[i]))
               return false;
         }
      }
      return true;
   }

   /**
    * query can be part of the key
    */
   public boolean matchesPartially(Query query) {
      if (query.parts.length > parts.length) {
         // query cannot be a part of the key
         return false;
      } else {

         // check each part
         for (int i = 0; i < query.parts.length; i++) {
            if (!regexMatches(query.parts[i], parts[i]))
               return false;
         }
         return true;
      }
   }

   public int getMatchingPrefixLength(Query query) {
      int matchingPrefixLength = 0;

      for (int i = 0; i < parts.length; i++) {
         if (!isPartWildcard(parts[i]) && regexMatches(query.parts[i], parts[i])) {
            matchingPrefixLength++;
         } else {
            return matchingPrefixLength;
         }
      }

      return matchingPrefixLength;
   }

   private static boolean isPartWildcard(String part) {
      return part.contains(WILDCARD);
   }

   private static String expandWildcard(String wildcard) {
      return wildcard.replaceAll("\\*", ".*");
   }

   private static boolean regexMatches(String s, String regex) {
      return s.matches(expandWildcard(regex));
   }
}
