/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.parameters.values;

import ikr.simlib.parameters.ParameterException;
import ikr.simlib.parameters.reflection.ReflectionConstructable;
import ikr.simlib.parameters.reflection.ReflectionConstructable1;
import ikr.simlib.parameters.reflection.ReflectionConstructable10;
import ikr.simlib.parameters.reflection.ReflectionConstructable2;
import ikr.simlib.parameters.reflection.ReflectionConstructable3;
import ikr.simlib.parameters.reflection.ReflectionConstructable4;
import ikr.simlib.parameters.reflection.ReflectionConstructable5;
import ikr.simlib.parameters.reflection.ReflectionConstructable6;
import ikr.simlib.parameters.reflection.ReflectionConstructable7;
import ikr.simlib.parameters.reflection.ReflectionConstructable8;
import ikr.simlib.parameters.reflection.ReflectionConstructable9;

import java.util.List;

public class Array extends Value {

   public final Value[] value;

   public Array(Value[] values) {
      this.value = values;
   }

   public Array(List<Value> values) {
      // create new array
      Value[] array = new Value[values.size()];

      // fill array
      int i = 0;
      for (Value v : values) {
         array[i] = v;
         i++;
      }

      this.value = array;
   }

   @Override
   public boolean asBoolean() {
      throw new ParameterException("Array cannot be interpreted as a boolean value.");
   }

   @Override
   public int asInteger() {
      throw new ParameterException("Array cannot be interpreted as an integer value.");
   }

   @Override
   public int[] asIntegerVector() {
      // create vector
      int[] vector = new int[value.length];

      // fill vector
      for (int i = 0; i < value.length; i++) {
         vector[i] = value[i].asInteger();
      }

      return vector;
   }

   @Override
   public int[][] asIntegerMatrix() {
      int[][] rows = new int[value.length][];

      // get row vectors
      for (int i = 0; i < value.length; i++) {
         rows[i] = value[i].asIntegerVector();
      }

      return rows;
   }

   @Override
   public String asString() {
      throw new ParameterException("Array cannot be interpreted as a string value.");
   }

   @Override
   public String[] asStringVector() {
      // create vector
      String[] vector = new String[value.length];

      // fill vector
      for (int i = 0; i < value.length; i++) {
         vector[i] = value[i].asString();
      }

      return vector;
   }

   @Override
   public String[][] asStringMatrix() {
      String[][] rows = new String[value.length][];

      // get row vectors
      for (int i = 0; i < value.length; i++) {
         rows[i] = value[i].asStringVector();
      }

      return rows;
   }

   @Override
   public <T extends Enum<T>> T asEnum(Class<T> enumClass) {
      throw new ParameterException("Array cannot be interpreted as an enum.");
   }

   @Override
   public <P0, P1, P2, P3, P4, P5, P6, P7, P8, P9, T extends ReflectionConstructable10<P0, P1, P2, P3, P4, P5, P6, P7, P8, P9>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6, P7 p7, P8 p8, P9 p9) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, P3, P4, P5, P6, P7, P8, T extends ReflectionConstructable9<P0, P1, P2, P3, P4, P5, P6, P7, P8>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6, P7 p7, P8 p8) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, P3, P4, P5, P6, P7, T extends ReflectionConstructable8<P0, P1, P2, P3, P4, P5, P6, P7>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6, P7 p7) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, P3, P4, P5, P6, T extends ReflectionConstructable7<P0, P1, P2, P3, P4, P5, P6>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5, P6 p6) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, P3, P4, P5, T extends ReflectionConstructable6<P0, P1, P2, P3, P4, P5>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, P3, P4, T extends ReflectionConstructable5<P0, P1, P2, P3, P4>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3, P4 p4) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, P3, T extends ReflectionConstructable4<P0, P1, P2, P3>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2, P3 p3) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, P2, T extends ReflectionConstructable3<P0, P1, P2>> T asInstance(Class<T> clazz, P0 p0, P1 p1, P2 p2) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, P1, T extends ReflectionConstructable2<P0, P1>> T asInstance(Class<T> clazz, P0 p0, P1 p1) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <P0, T extends ReflectionConstructable1<P0>> T asInstance(Class<T> clazz, P0 p0) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public <T extends ReflectionConstructable> T asInstance(Class<T> clazz) {
      throw new ParameterException("Array cannot be interpreted as a class description.");
   }

   @Override
   public double asDouble() {
      throw new ParameterException("Array cannot be interpreted as a double value.");
   }

   @Override
   public double[] asDoubleVector() {
      // create vector
      double[] vector = new double[value.length];

      // fill vector
      for (int i = 0; i < value.length; i++) {
         vector[i] = value[i].asDouble();
      }

      return vector;
   }

   @Override
   public double[][] asDoubleMatrix() {
      double[][] rows = new double[value.length][];

      // get row vectors
      for (int i = 0; i < value.length; i++) {
         rows[i] = value[i].asDoubleVector();
      }

      return rows;
   }

   @Override
   public long asLong() {
      throw new ParameterException("Array cannot be interpreted as a long value.");
   }

   @Override
   public long[] asLongVector() {
      // create vector
      long[] vector = new long[value.length];

      // fill vector
      for (int i = 0; i < value.length; i++) {
         vector[i] = value[i].asLong();
      }

      return vector;
   }

   @Override
   public long[][] asLongMatrix() {
      long[][] rows = new long[value.length][];

      // get row vectors
      for (int i = 0; i < value.length; i++) {
         rows[i] = value[i].asLongVector();
      }

      // TODO: check if row lengths vary

      return rows;
   }
   
   @Override
   public String toString(){
      StringBuilder ret = new StringBuilder("[ ");
      for (int i = 0; i < value.length; i++) {
         ret.append(value[i].toString());
         if(i < value.length -1)
            ret.append(" ");
      }
      ret.append(" ]");
      return ret.toString();
   }

}
