/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.ports.input;

import ikr.simlib.entities.Entity;
import ikr.simlib.messages.Message;
import ikr.simlib.ports.Port;
import ikr.simlib.ports.PortException;
import ikr.simlib.ports.output.OutputPort;

public abstract class InputPort extends Port {

   public static final String INPUT_PORT_NAME = "input";

   protected OutputPort prev;

   public InputPort(Entity owner, String portName) {
      super(owner, portName);
   }

   public InputPort(Entity owner) {
      this(owner, INPUT_PORT_NAME);
   }

   @Override
   public Port getConnectedPort() {
      return prev;
   }

   public OutputPort getConnectedOutputPort() {
      return prev;
   }

   public void setConnectedOutputPort(OutputPort fromPort) {
      prev = fromPort;
   }

   @Override
   protected boolean handleIsMessageAvailable() {
      if (prev != null) {
         // call connected output port
         return prev.isMessageAvailable();

      } else {
         throw new PortException("IsMessageAvailable() was not handled at port: " + getName() + ".");
      }
   }

   @Override
   protected Message handleGetMessage() {
      if (prev != null) {
         // call the connected output port
         return prev.getMessage();

      } else {
         // nobody handled it, throw port exception
         throw new PortException("GetMessage() was not handled at port: " + getName() + ".");
      }
   }
}
