/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.conditional.implementations;

import ikr.simlib.model.SimNode;
import ikr.simlib.statistics.conditional.ConditionalSampleCollectorBase;
import ikr.simlib.statistics.conditional.ConditionalUnweightedSampleCollectorBase;
import ikr.simlib.statistics.sample.StdSampleStatistic;
import ikr.simlib.statistics.support.BucketUtility;

public class ConditionalStdSampleStatistic<K> extends ConditionalUnweightedSampleCollectorBase<K, Double, StdSampleStatistic> {

   private final static StatisticCreator<StdSampleStatistic> CONSTRUCTOR = new StatisticCreator<StdSampleStatistic>() {
      @Override
      public StdSampleStatistic createNewStatistic(SimNode ownNode) {
         return new StdSampleStatistic(ownNode);
      }
   };

   public ConditionalStdSampleStatistic(SimNode ownNode, Class<K> keyClass, BucketUtility bucketUtility) {
      super(ownNode, keyClass, CONSTRUCTOR, bucketUtility);
   }

   public ConditionalStdSampleStatistic(SimNode ownNode, Class<K> keyClass, int numBuckets) {
      super(ownNode, keyClass, CONSTRUCTOR, numBuckets);
   }

   public ConditionalStdSampleStatistic(SimNode ownNode, Class<K> keyClass, Iterable<? extends K> keys, StatisticTagger<? super K, ? super StdSampleStatistic> tagger) {
      super(ownNode, keyClass, CONSTRUCTOR, keys, tagger);
   }

   public ConditionalStdSampleStatistic(SimNode ownNode, Class<K> keyClass, Iterable<? extends K> keys) {
      super(ownNode, keyClass, CONSTRUCTOR, keys, ConditionalSampleCollectorBase.TO_STRING_TAGGER);
   }
}
