/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.statistics.support;

import ikr.simlib.exceptions.SimulationException;

public class GeoBucketUtility extends BucketUtility {

   public static double DEFAULT_BASE = 0;

   private final double minBucketWidth;
   private final double base;
   private final double logBase;

   public GeoBucketUtility(double base, double min, double max, int noOfBuckets) {
      super(min, max, noOfBuckets);

      if (base <= 1) {
         throw new SimulationException("base is <= 1");
      }

      this.base = base;
      minBucketWidth = (max - min) * (base - 1) / (Math.pow(base, noOfBuckets) - 1);
      logBase = Math.log(base);
   }

   @Override
   public int getBucket(double condVal) {
      if (!isInRange(condVal)) {
         throw new SimulationException("Condition value out of range");
      }

      return (int) (Math.floor(Math.log((condVal / minBucketWidth) * (base - 1) + 1) / logBase));
   }

   @Override
   public double getLowerBound(int index) {
      if (index >= getArraySize()) {
         throw new SimulationException("index out of range");
      }

      return getMin() + minBucketWidth * (Math.pow(base, index) - 1) / (base - 1);
   }

   @Override
   public double getUpperBound(int index) {
      if (index >= getArraySize()) {
         throw new SimulationException("index out of range");
      }

      return getMin() + minBucketWidth * (Math.pow(base, index + 1) - 1) / (base - 1);
   }

   public double getMinBucketWidth() {
      return minBucketWidth;
   }

   public double getBase() {
      return base;
   }

   public double getLogBase() {
      return logBase;
   }
}
