/*
 * Copyright (C) 2007-2018 University of Stuttgart, IKR
 * 
 * This file is part of IKR SimLib, Java Edition, as available from
 * http://www.ikr.uni-stuttgart.de/IKRSimLib.
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License (LGPL) as published by the
 * Free Software Foundation, in version 2.1 as it comes in the file "COPYING" in
 * directory doc of IKR SimLib.
 * 
 * IKR SimLib is distributed in the hope that it will be useful, but without any
 * warranty of any kind. See the LGPL for details or contact the IKR if you need
 * additional information (http://www.ikr.uni-stuttgart.de).
 * 
 * Alternatively you may license IKR SimLib under a different (non-free)
 * license. Please contact the IKR for further details.
 */

package ikr.simlib.support.compression;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.OutputStream;

public interface CompressionProvider {

   /**
    * Opens the given file for reading.
    * 
    * @param file
    *           the file to open, with extension at it is on the disk
    * @return an InputStream to read the file
    * @throws FileNotFoundException
    *            when the file is not readable
    */
   public InputStream getInputStream(File file) throws FileNotFoundException;

   /**
    * Opens the given file for writing.
    * 
    * @param file
    *           the file to open, whith extension as it shall be created
    * @return an OutputStream to write to this file
    * @throws FileNotFoundException
    *            when the file cannot be created
    */
   public OutputStream getOutputStream(File file) throws FileNotFoundException;
}
